/*
 *
 * happytimer - Alarm clock for the GNOME
 *
 *  @Copyright (C) 2007-2011 JinCheol Kim (shriekout@gmail.com).
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <config.h>
#include "happytimer.h"
#include "interface.h"
#include "timecheck.h"

void destroy(GtkWidget *widget, gpointer data)
{
	gtk_main_quit();
}

void display_timer(void)
{
	static gboolean window_shown = FALSE;

	if (!window_shown) {
		gtk_widget_show(timer_window);
	} else {
		gtk_widget_hide(timer_window);
	}

	window_shown = !window_shown;
}

void display_preferences(GtkWidget *widget, gpointer data)
{
	if (!preferences_dialog) {
		preferences_dialog = create_preferencesdialog();
		gtk_dialog_run(GTK_DIALOG(preferences_dialog));

		gtk_widget_destroy(preferences_dialog);
		preferences_dialog = NULL;
		rtplay_togglebutton = NULL;
		tplay_togglebutton = NULL;
	}
}

void display_about_dialog(GtkWidget *widget, gpointer data)
{
	const gchar *artists[] = {_("JinCheol Kim<shriekout@gmail.com>"), _("BunDo Kang<bundo@bundo.biz>"), _("SooIk Lee<iggy27@naver.com>"), _("Arnold W. Shim <awshim@nate.com>"),NULL};
	const gchar *authors[] = {_("JinCheol Kim <shriekout@gmail.com>"), NULL};
	const gchar *license ={_("happytimer is free software: you can redistribute it and/or modify\n"
				"it under the terms of the GNU General Public License as published by\n"
				"the Free Software Foundation, either version 3 of the License, on"
				"(at your option) any later version.\n"
				"\n"
				"happytimer is distributed in the hope that it will be useful,\n"
				"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
				"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
				"GNU General Public License for more details.\n"
				"\n"
				"You should have received a copy of the GNU General Public License\n"
				"along with this program.  If not, see <http://www.gnu.org/licenses/>.")};
	GdkPixbuf *pix;

	pix = gdk_pixbuf_new_from_file(LOGO_FILE, NULL);

	gtk_show_about_dialog(NULL,
			"artists", artists,
			"authors", authors,
			"comments", _("Alarm clock for the GNOME."),
			"copyright", _("\xc2\xa9 2007–2011 JinCheol Kim<shriekout@gmail.com>"),
			"license", license,
			"logo", pix,
			"version", PACKAGE_VERSION,
			"program-name", _("Happytimer"),
			"title", _("about Happytimer"),
			"website", "http://kldp.net/projects/happytimer",
			NULL);

	gdk_pixbuf_unref(pix);
}

gboolean on_icon_button_press_event(GtkStatusIcon *icon, GdkEventButton *event, gpointer data)
{
	switch (event->button) {
		case 1: display_timer();
			break;
		case 3: gtk_menu_popup(GTK_MENU(timer_menu), NULL,
					NULL, gtk_status_icon_position_menu, (gpointer)icon,
					event->button, event->time);
			break;
		default:
			break;
	}

	return TRUE;
}

static void set_timer_spin(void)
{
	struct struct_t *buf;

	buf = return_struct_date((time_t)varList.timer_origin);

	gtk_spin_button_set_value(GTK_SPIN_BUTTON(hrs_spinbutton), (gdouble)buf->hrs);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(mins_spinbutton), (gdouble)buf->mins);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(secs_spinbutton), (gdouble)buf->secs);

	free(buf);
}

void on_minus_30_button_clicked(GtkWidget *widget, gpointer data)
{
	varList.timer_origin -= 1800;

	if (varList.timer_origin < 0)
		varList.timer_origin = 0;

	set_timer_spin();
}

void on_plus_30_button_clicked(GtkWidget *widget, gpointer data)
{
	varList.timer_origin += 1800;

	if (varList.timer_origin > MAX_TIMER)
		varList.timer_origin = MAX_TIMER;

	set_timer_spin();
}

void timer_spinbuttons_value_changed(GtkWidget *widget, gpointer data)
{
	gchar *buf;
	char buf1[80];

	varList.timer_origin = gtk_spin_button_get_value(GTK_SPIN_BUTTON(hrs_spinbutton)) * 3600;
	varList.timer_origin += gtk_spin_button_get_value(GTK_SPIN_BUTTON(mins_spinbutton)) * 60;
	varList.timer_origin += gtk_spin_button_get_value(GTK_SPIN_BUTTON(secs_spinbutton));

	buf = return_format_date(varList.timer_origin);
	
	gtk_label_set_text(GTK_LABEL(timerleft_label), buf);

	g_free(buf);

	sprintf(buf1, "%d", varList.timer_origin);

	save_preferences("ptime", buf1, NULL, NULL, NULL, NULL, EDIT);
}

static void widget_set_activate(gboolean s)
{
	gtk_widget_set_sensitive(minus_30_button, s);
	gtk_widget_set_sensitive(plus_30_button, s);
	gtk_widget_set_sensitive(hrs_spinbutton, s);
	gtk_widget_set_sensitive(mins_spinbutton, s);
	gtk_widget_set_sensitive(secs_spinbutton, s);
	gtk_widget_set_sensitive(preset_comboboxtext, s);
	gtk_widget_set_sensitive(pause_button, !s);
}

void on_start_button_clicked(GtkWidget *widget, gpointer data)
{
	char *buf;

	varList.counting = !varList.counting;

	if (varList.counting) {
		varList.count = varList.timer_origin;
		gtk_image_set_from_stock(GTK_IMAGE(start_button_image), "gtk-media-stop", GTK_ICON_SIZE_BUTTON);
		gtk_widget_set_tooltip_text(start_button, _("Stop"));

		widget_set_activate(FALSE);
		gtk_status_icon_set_from_file(happytimer_statusicon, GREEN_LOGO_FILE);
	} else {
		gtk_image_set_from_stock(GTK_IMAGE(start_button_image), "gtk-media-play", GTK_ICON_SIZE_BUTTON);
		gtk_widget_set_tooltip_text(start_button, _("Start"));

		widget_set_activate(TRUE);
		gtk_status_icon_set_from_file(happytimer_statusicon, LOGO_FILE);

		buf = return_format_date(varList.timer_origin);
		gtk_label_set_text(GTK_LABEL(timerleft_label), buf);
		g_free(buf);

		varList.pause = FALSE;
	}
}

void on_pause_button_clicked(GtkWidget *widget, gpointer data)
{
	gchar *tmp;
	char buf[80];

	varList.pause = !varList.pause;

	if (varList.pause) {
		tmp = return_format_date(varList.count);
		sprintf(buf, _("Pause! - %s"), tmp);
		g_free(tmp);

		gtk_label_set_text(GTK_LABEL(timerleft_label), buf);
		gtk_status_icon_set_tooltip_text(happytimer_statusicon, buf);

		gtk_status_icon_set_from_file(happytimer_statusicon, PAUSE_IMAGE_FILE);
	} else {
		gtk_status_icon_set_from_file(happytimer_statusicon, GREEN_LOGO_FILE);
	}
}

void alarm_window_destroy(GtkWidget *widget)
{
	gtk_widget_destroy(alarm_window);
	alarm_window = NULL;


	if (varList.pipeline[ARM_SND] != NULL) {
		gst_element_set_state(varList.pipeline[ARM_SND], GST_STATE_NULL);
		g_object_unref(varList.pipeline[ARM_SND]);
		varList.pipeline[ARM_SND] = NULL;
	}
}

void on_rtsound_checkbutton_toggled(GtkWidget *widget, gpointer data)
{
	char buf[80];

	gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
	prefer.rtsound = active;

	sprintf(buf, "%d", prefer.rtsound);
	save_preferences("rtsound", "switch", buf, NULL, NULL, NULL, EDIT);
}

void on_tsound_checkbutton_toggled(GtkWidget *widget, gpointer data)
{
	char buf[80];

	gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
	prefer.tsound = active;

	sprintf(buf, "%d", prefer.tsound);
	save_preferences("tsound", "switch", buf, NULL, NULL, NULL, EDIT);
}

void sound_update_preview_cb(GtkFileChooser *fc, gpointer data)
{
	char *fn = gtk_file_chooser_get_preview_filename(fc);

	if (!fn)
		return;

	if (varList.pipeline[PRE_SND]) {
		gst_element_set_state(GST_ELEMENT(varList.pipeline[PRE_SND]), GST_STATE_NULL);
		g_object_unref(varList.pipeline[PRE_SND]);
		varList.pipeline[PRE_SND] = NULL;
	}

	play_sound(PRE_SND, fn);

	g_free(fn);
}

void on_repeat_checkbutton_toggled(GtkWidget *widget, gpointer data)
{
	gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
	varList.repeat = active;
}

void on_preview_stop_button_clicked(GtkWidget *widget, gpointer data)
{
	if (varList.pipeline[PRE_SND]) {
		gst_element_set_state(GST_ELEMENT(varList.pipeline[PRE_SND]), GST_STATE_NULL);
		g_object_unref(varList.pipeline[PRE_SND]);
		varList.pipeline[PRE_SND] = NULL;
	}
}

void rtsound_file_set_cb(GtkFileChooser *fc, gpointer data)
{
	if (varList.pipeline[PRE_SND]) {
		gst_element_set_state(GST_ELEMENT(varList.pipeline[PRE_SND]), GST_STATE_NULL);
		g_object_unref(varList.pipeline[PRE_SND]);
		varList.pipeline[PRE_SND] = NULL;
	}

	char *fn = gtk_file_chooser_get_filename(fc);

	free(prefer.sound_file[RT_SND]);
	prefer.sound_file[RT_SND] = strdup(fn);
	g_free(fn);

	save_preferences("rtsound", NULL, NULL, prefer.sound_file[RT_SND], NULL, NULL, EDIT);
}

void tsound_file_set_cb(GtkFileChooser *fc, gpointer data)
{
	if (varList.pipeline[PRE_SND]) {
		gst_element_set_state(GST_ELEMENT(varList.pipeline[PRE_SND]), GST_STATE_NULL);
		g_object_unref(varList.pipeline[PRE_SND]);
		varList.pipeline[PRE_SND] = NULL;
	}

	char *fn = gtk_file_chooser_get_filename(fc);

	free(prefer.sound_file[ARM_SND]);
	prefer.sound_file[ARM_SND] = strdup(fn);
	g_free(fn);

	save_preferences("tsound", NULL, NULL, prefer.sound_file[ARM_SND], NULL, NULL, EDIT);
}

void on_rtplay_togglebutton_toggled(GtkWidget *widget, gpointer data)
{
	gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));

	if (active) {
		play_sound(RT_SND, NULL);
	} else {
		if (varList.pipeline[RT_SND]) {
			gst_element_set_state(GST_ELEMENT(varList.pipeline[RT_SND]), GST_STATE_NULL);
			g_object_unref(varList.pipeline[RT_SND]);
			varList.pipeline[RT_SND] = NULL;
		}
	}
}

void on_tplay_togglebutton_toggled(GtkWidget *widget, gpointer data)
{
	gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));

	if (active) {
		play_sound(ARM_SND, NULL);
	} else {
		if (varList.pipeline[ARM_SND]) {
			gst_element_set_state(GST_ELEMENT(varList.pipeline[ARM_SND]), GST_STATE_NULL);
			g_object_unref(varList.pipeline[ARM_SND]);
			varList.pipeline[ARM_SND] = NULL;
		}
	}
}

static void update_image_preview_cb(GtkFileChooser *file_chooser, gpointer data)
{
	GtkWidget *preview;
	char *filename;
	GdkPixbuf *pixbuf;
	gboolean have_preview;

	preview = GTK_WIDGET(data);
	filename = gtk_file_chooser_get_preview_filename(file_chooser);

	if (filename == NULL)
		return;

	pixbuf = gdk_pixbuf_new_from_file_at_size(filename, 128, 128, NULL);
	have_preview = (pixbuf != NULL);
	g_free (filename);

	gtk_image_set_from_pixbuf(GTK_IMAGE(preview), pixbuf);
	if (pixbuf)
		g_object_unref (pixbuf);

	gtk_file_chooser_set_preview_widget_active (file_chooser, have_preview);
}

void on_image_chooser_button(GtkWidget *widget, gpointer data)
{
	GtkWidget *dialog;
	GtkWidget *preview = NULL;
	GtkFileFilter *image_filter, *all_filter;
	GdkPixbuf *pixbuf;
	char *image_mime_types[] = {
		"image/png",
		"image/jpeg",
		"image/gif"
	};
	char *image_pattern[] = {
		"*.png",
		"*.jpg",
		"*.gif",
		"*.tif",
		"*.xpm"
	};
	int count;

	dialog = gtk_file_chooser_dialog_new(_("Open File"),
					NULL,
					GTK_FILE_CHOOSER_ACTION_OPEN,
					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
					NULL);

	preview = gtk_image_new();

	gtk_file_chooser_set_preview_widget(GTK_FILE_CHOOSER(dialog), preview);
	g_signal_connect(G_OBJECT(dialog), "update-preview",
			G_CALLBACK (update_image_preview_cb), preview);

	image_filter = gtk_file_filter_new();
	gtk_file_filter_set_name(image_filter, _("Image Files"));

	for (count = 0; count < G_N_ELEMENTS(image_mime_types); count++)
		gtk_file_filter_add_mime_type(image_filter, image_mime_types[count]);

	for (count = 0; count < G_N_ELEMENTS(image_pattern); count++)
		gtk_file_filter_add_pattern(image_filter, image_pattern[count]);

	all_filter = gtk_file_filter_new();
	gtk_file_filter_set_name(all_filter, _("All files"));
	gtk_file_filter_add_pattern(all_filter, "*");

	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), image_filter);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), all_filter);

	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		char *filename;

		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));

		free(prefer.timage_file);
		prefer.timage_file = strdup(filename);

		pixbuf = gdk_pixbuf_new_from_file_at_size(prefer.timage_file, 128, 128, NULL);
		gtk_image_set_from_pixbuf(GTK_IMAGE(preview_image), pixbuf);
		gdk_pixbuf_unref(pixbuf);

		g_free(filename);

		save_preferences("arm_image", NULL, NULL, prefer.timage_file, NULL, NULL, EDIT);
	}

	gtk_widget_destroy (dialog);
}

void on_notify_spinbutton_value_changed(GtkWidget *widget, gpointer data)
{
	char buf[80];
	int temp;

	temp = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
	prefer.notify_before = temp * 60;
	sprintf(buf, "%d", prefer.notify_before);

	save_preferences("notify", NULL, NULL, buf, NULL, NULL, EDIT);
}

void on_preset_comboboxtext_changed(GtkWidget *widget, gpointer data)
{
	int active;
	int count;
	struct presetList *cur;
	struct struct_t *date = NULL;

	active = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));

	cur = presetList_head;	

	for (count = 0; count < active; count++)
		cur = cur->next;

	date = return_struct_date((time_t)cur->timer);

	gtk_spin_button_set_value(GTK_SPIN_BUTTON(hrs_spinbutton), (gdouble)date->hrs);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(mins_spinbutton), (gdouble)date->mins);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(secs_spinbutton), (gdouble)date->secs);

	free(date);
}

void on_add_preset_button_clicked(GtkWidget *widget, gpointer data)
{
	GtkBuilder *builder;
	GtkWidget *dialog;
	GtkWidget *preset_name;
	GtkWidget *hrs_widget;
	GtkWidget *mins_widget;
	GtkWidget *secs_widget;
	GError *error = NULL;
	int result;
	const char *name;
	int timer;
	struct presetList *cur = NULL, *new = NULL;
	char *buf, temp[80];

	builder = gtk_builder_new();
	if (!gtk_builder_add_from_file(builder, UI_DIR"/add_edit_preset_dialog.glade", &error)) {
		g_warning(_("Could't load builder file: %s"), error->message);
		g_error_free(error);
	}

	gtk_builder_connect_signals(builder, NULL);
	dialog = GTK_WIDGET(gtk_builder_get_object(builder, "add_edit_preset_dialog"));
	gtk_window_set_title(GTK_WINDOW(dialog), _("Add preset"));
	preset_name = GTK_WIDGET(gtk_builder_get_object(builder, "preset_entry"));
	hrs_widget = GTK_WIDGET(gtk_builder_get_object(builder, "preset_hrs_spinbutton"));
	mins_widget = GTK_WIDGET(gtk_builder_get_object(builder, "preset_mins_spinbutton"));
	secs_widget = GTK_WIDGET(gtk_builder_get_object(builder, "preset_secs_spinbutton"));

	gtk_dialog_add_buttons(GTK_DIALOG(dialog),
			GTK_STOCK_ADD,
			GTK_RESPONSE_OK,
			GTK_STOCK_CLOSE,
			GTK_RESPONSE_CLOSE,
			NULL);

	result = gtk_dialog_run(GTK_DIALOG(dialog));

	if ((result == GTK_RESPONSE_OK) && strlen(name = gtk_entry_get_text(GTK_ENTRY(preset_name)))) {
		timer = gtk_spin_button_get_value(GTK_SPIN_BUTTON(hrs_widget)) * 3600;
		timer += (gtk_spin_button_get_value(GTK_SPIN_BUTTON(mins_widget)) * 60);
		timer += gtk_spin_button_get_value(GTK_SPIN_BUTTON(secs_widget));

		cur = presetList_head;

		while (cur->next)
			cur = cur->next;

		new = (struct presetList *)malloc(sizeof(struct presetList));
		new->name = strdup(name);
		new->timer = timer;
		new->next = NULL;
		cur->next = new;

		buf = return_format_date((time_t)new->timer);
		sprintf(temp, "%s (%s)", new->name, buf);
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(preset_comboboxtext), temp);
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(preset_config_comboboxtext), temp);
		free(buf);

		sprintf(temp, "%d", new->timer);
		save_preferences("preset", "name", new->name, temp, NULL, NULL, PRESET_ADD);
	}

	gtk_widget_destroy(dialog);
}

void on_edit_preset_button_clicked(GtkWidget *widget, gpointer data)
{
	GtkBuilder *builder;
	GtkWidget *dialog;
	GtkWidget *preset_name;
	GtkWidget *hrs_widget;
	GtkWidget *mins_widget;
	GtkWidget *secs_widget;
	GError *error = NULL;
	int result;
	int active;
	int count;
	struct presetList *cur = NULL;
	struct struct_t *date;
	char *buf;
	const char *name;
	int timer;
	char temp[80], temp2[80];

	builder = gtk_builder_new();
	if (!gtk_builder_add_from_file(builder, UI_DIR"/add_edit_preset_dialog.glade", &error)) {
		g_warning(_("Could't load builder file: %s"), error->message);
		g_error_free(error);
	}

	gtk_builder_connect_signals(builder, NULL);
	dialog = GTK_WIDGET(gtk_builder_get_object(builder, "add_edit_preset_dialog"));
	gtk_window_set_title(GTK_WINDOW(dialog), _("Edit preset"));
	preset_name = GTK_WIDGET(gtk_builder_get_object(builder, "preset_entry"));
	hrs_widget = GTK_WIDGET(gtk_builder_get_object(builder, "preset_hrs_spinbutton"));
	mins_widget = GTK_WIDGET(gtk_builder_get_object(builder, "preset_mins_spinbutton"));
	secs_widget = GTK_WIDGET(gtk_builder_get_object(builder, "preset_secs_spinbutton"));

	gtk_dialog_add_buttons(GTK_DIALOG(dialog),
			GTK_STOCK_EDIT,
			GTK_RESPONSE_OK,
			GTK_STOCK_CLOSE,
			GTK_RESPONSE_CLOSE,
			NULL);

	active = gtk_combo_box_get_active(GTK_COMBO_BOX(preset_config_comboboxtext));

	if (presetList_head == NULL) {
		return;
	}

	cur = presetList_head;

	for (count = 0; count < active; count++) {
		cur = cur->next;
	}

	gtk_entry_set_text(GTK_ENTRY(preset_name), cur->name);

	date = return_struct_date((time_t)cur->timer);

	gtk_spin_button_set_value(GTK_SPIN_BUTTON(hrs_widget), (gdouble)date->hrs);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(mins_widget), (gdouble)date->mins);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(secs_widget), (gdouble)date->secs);

	free(date);

	result = gtk_dialog_run(GTK_DIALOG(dialog));

	if ((result == GTK_RESPONSE_OK) && strlen(name = gtk_entry_get_text(GTK_ENTRY(preset_name)))) {
		timer = gtk_spin_button_get_value(GTK_SPIN_BUTTON(hrs_widget)) * 3600;
		timer += (gtk_spin_button_get_value(GTK_SPIN_BUTTON(mins_widget)) * 60);
		timer += gtk_spin_button_get_value(GTK_SPIN_BUTTON(secs_widget));

		sprintf(temp, "%d", timer);
		sprintf(temp2, "%d", cur->timer);
		save_preferences("preset", "name", (char *)name, temp, cur->name, temp2, PRESET_EDIT);

		free(cur->name);
		cur->name = strdup(name);
		cur->timer = timer;

		buf = return_format_date((time_t)cur->timer);
		sprintf(temp, "%s (%s)", cur->name, buf);
		free(buf);

		gtk_combo_box_text_remove(GTK_COMBO_BOX_TEXT(preset_comboboxtext), active);
		gtk_combo_box_text_insert(GTK_COMBO_BOX_TEXT(preset_comboboxtext), active, NULL, temp);
		gtk_combo_box_text_remove(GTK_COMBO_BOX_TEXT(preset_config_comboboxtext), active);
		gtk_combo_box_text_insert(GTK_COMBO_BOX_TEXT(preset_config_comboboxtext), active, NULL, temp);		
	}

	gtk_widget_destroy(dialog);
}

void on_remove_preset_button_clicked(GtkWidget *widget, gpointer data)
{
	GtkWidget *dialog;
	int active;
	int result;
	int count;
	char *buf, temp[80];
	struct presetList *cur = NULL, *del;

	active = gtk_combo_box_get_active(GTK_COMBO_BOX(data));

	if (presetList_head == NULL) {
		return;
	}

	del = presetList_head;

	for (count = 0; count < active; count++) {
		cur = del;
		del = del->next;
	}

	buf = return_format_date((time_t)del->timer);

	dialog = gtk_message_dialog_new(NULL,
					GTK_DIALOG_DESTROY_WITH_PARENT,
					GTK_MESSAGE_QUESTION,
					GTK_BUTTONS_OK_CANCEL,
					_("Are you really want to remove this '%s (%s)'?"),
					del->name, buf);

	free(buf);

	result = gtk_dialog_run (GTK_DIALOG (dialog));

	if (result == GTK_RESPONSE_OK) {
		gtk_combo_box_text_remove(GTK_COMBO_BOX_TEXT(data), active);
		gtk_combo_box_set_active(GTK_COMBO_BOX(data), 0);
		gtk_combo_box_text_remove(GTK_COMBO_BOX_TEXT(preset_comboboxtext), active);
		gtk_combo_box_set_active(GTK_COMBO_BOX(preset_comboboxtext), 0);

		sprintf(temp, "%d", del->timer);
		save_preferences("preset", "name", NULL, NULL, del->name, temp, PRESET_DELETE);

		if (active) {
			cur->next = del->next;
		} else {
			presetList_head = del->next;
		}

		free(del->name);
		free(del);
	}

	gtk_widget_destroy (dialog);
}

void stop_sound_cb(GtkWidget *widget, gpointer data)
{
	int count;

	for (count = 0; count < SND_NUM; count++) {
		if (varList.pipeline[count]) {
			gst_element_set_state(GST_ELEMENT(varList.pipeline[count]), GST_STATE_NULL);
			g_object_unref(varList.pipeline[count]);
			varList.pipeline[count] = NULL;
		}
	}
}
